#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Should be imported as:
# 
#

# _____________________________________________________________________
#
# Imports 
#   * imported by import order.
# 	* keep full writen due to import order.
# 

# imports .
from dev0s.classes import exceptions
from dev0s.classes import utils
from dev0s.classes import color
from dev0s.classes import console
from dev0s.classes import env
from dev0s.classes import files
from dev0s.classes import response
from dev0s.classes import objects
from dev0s.classes import defaults
from dev0s.classes import cli
from dev0s.classes import requests as _requests_
from dev0s.classes import code

# _____________________________________________________________________
#
# Shortcuts (also in order):
#   * should be imported as:
#       from dev0s import *
#

# exceptions.
Exceptions = exceptions.Exceptions

# color.
symbol = color.symbol
color = color.color

# console.
Console = console.Console
Loader = Console.Loader
ProgressLoader = Console.ProgressLoader

# environment.
Environment = env.Environment

# files & formats.
gfp = files.gfp
gdate = files.gdate
Files = files.Files
Formats = files.Formats
Generate = Formats.Generate
FilePath = Formats.FilePath 
String = Formats.String 
Boolean = Formats.Boolean 
Integer = Formats.Integer 
Date = Formats.Date
File = Files.File
Directory = Files.Directory
Zip = Files.Zip
Image = Files.Image
Bytes = Files.Bytes
Dictionary = Files.Dictionary
Array = Files.Array

# objects.
Traceback = objects.Traceback
Object = objects.Object
Thread = objects.Thread

# response.
Response = response.Response
ResponseObject = Response.ResponseObject

# defaults.
Defaults = defaults.Defaults

# cli.
CLI = cli
CLI.CLI = CLI.CLI

# requests.
Requests = _requests_.Requests
RestAPI = Requests.RestAPI

# code.
Code = code.Code
execute = Code.execute
Spawn = Code.Spawn
OutputObject = Code.OutputObject
Version = Code.Version 
Script = Code.Script
Python = Code.Python

# _____________________________________________________________________
#
# Universal:
#   * universal package imports (do not edit).
#

# source, base & version.
source = Directory(gfp.base(__file__))
base = Directory(source.fp.base())
try: version = Version(Files.load(source.join(".version")))
except: version = None

#


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from dev0s.classes.config import *

# the docs object class.
# used to assign unique readme building values to a coded class.
class Docs(object):
	def __init__(self,
		# boolean inidicating if the object is initialized by default.
		initialized=True,
		# the full module path in import style (when initializing).
		module="Code.Docs",
		# the notes that will apread above the class_ initialization (leave [] to use default.
		notes=[],
	):
		
		# attributes.
		self.initialized = initialized
		self.module = module
		self.notes = notes

		# checks.
		if self.notes in [None, False, ""]: self.notes = []

		#




# imports
from dev0s.classes.config import *
from dev0s.classes.color import color, symbol
import getpass
_input_ = input

# the console class.
class Console():

	# input.
	def input(message, yes_no=False, check=False, password=False, default=None):
		message = color.fill(message)
		if len(message) > 0 and message[0].upper() != message[0]: message = String(message).capitalized_word()
		if yes_no:
			while True:
				if len(message) > 0 and message[len(message)-1] == " ": message = message[:-1]
				elif len(message) > 0 and message[len(message)-1] == ".": message = message[:-1]
				else: break
			value = _input_(message+" (y/n): ")
			if value in ["yes", 'y', "YES", "Y"]:
				return True
			else:
				return False
		else:
			while True:
				if len(message) > 0 and message[len(message)-1] == " ": message = message[:-1]
				elif len(message) > 0 and message[len(message)-1] == ":": message = message[:-1]
				else: break
			if password:
				value = getpass.getpass(message+": ")
			else:
				value = _input_(message+": ")
				if value in [""]: value = default
			if check:
				if password:
					if getpass.getpass("Enter the same passphrase: ") != password:
						print("Passphrases do not match.")
						return default
					else:
						return password
				else:
					if Console.input(f"({value}) correct?", yes_no=True):
						return value
					else:
						return default
			else:
				return value

	# the loader object class.
	class Loader(threading.Thread):
		def __init__(self, message, autostart=True, log_level=0, interactive=True):
			threading.Thread.__init__(self)
			self.message = self.__clean_message__(message)
			self.last_message = str(self.message)
			self.log_level = log_level
			self.interactive = interactive
			if autostart and self.log_level >= 0: 
				if self.interactive:
					try:
						self.start()
					except KeyboardInterrupt as e:
						self.stop(success=False)
						raise KeyboardInterrupt(f"{e}")
				else:
					print(self.message+".")
		def run(self):
			if self.log_level >= 0: 
				self.running = True
				self.released = True
				while self.running:
					if not self.released:
						time.sleep(1)
					else:
						for i in ["|", "/", "-", "\\"]:
							if not self.released: break
							if self.message != self.last_message:
								print(self.__empty_message__(length=len(f"{self.last_message} ...   ")), end="\r")
								self.message = self.__clean_message__(self.message)
							print(f"{self.message} ... {i}", end="\r")
							self.last_message = self.message
							if not self.running: break
							time.sleep(0.33)
			self.running = "stopped"
		def stop(self, message=None, success=True, response=None, quiet=False):
			if self.log_level >= 0:
				if response == None:
					if message == None: message = self.message
				else:
					if response["error"] == None:
						message = response["message"]
					else:
						success = False
						message = "Error: "+response["error"]
				if self.interactive:
					self.running = False
					for i in range(120):
						if self.running == "stopped": break
						time.sleep(0.5)
					if self.running != "stopped": raise ValueError(f"Unable to stop loader [{self.message}].")
				if not quiet:
					if self.interactive:
						print(self.__empty_message__(length=len(f"{self.last_message} ...   ")), end="\r")
						if success:
							print(f"{message} ... done")
						else:
							print(f"{message} ... {color.red}failed{color.end}")
					else:
						if success:
							print(f"{message}. done")
						else:
							print(f"{message}. {color.red}failed{color.end}")
		def mark(self, new_message=None, old_message=None, success=True, response=None):
			if self.log_level >= 0: 
				if response != None:
					if response["error"] == None:
						success = True
					else:
						success = False
				if old_message == None: old_message = self.message
				if self.interactive:
					print(self.__empty_message__(length=len(f"{self.last_message} ...   ")), end="\r")
					if success:
						print(f"{old_message} ... done")
					else:
						print(f"{old_message} ... {color.red}failed{color.end}")
				else:
					if success:
						print(f"{old_message}. done")
					else:
						print(f"{old_message}. {color.red}failed{color.end}")
				if new_message != None: self.message = new_message
		def hold(self):
			if self.log_level >= 0: 
				self.released = False
				time.sleep(0.33)
		def release(self):
			if self.log_level >= 0: 
				self.released = True
				time.sleep(0.33)
		# system functions.
		def __clean_message__(self, message):
			message = color.fill(str(message))
			if message[-len(" ..."):] == " ...": message = message[:-4]
			if message[-len("."):] == ".": message = message[:-1]
			if message[0].upper() != message[0]: message = message[1:]+message[0].upper()+message[1:]
			return message
		def __empty_message__(self, length=len("hello world")):
			s = ""
			for i in range(length): s += " "
			return s

	# the loader object class.
	class ProgressLoader(threading.Thread):
		def __init__(self, message, index=0, max=10, log_level=0):
			threading.Thread.__init__(self)
			self.message = message
			if self.message[-len(" ..."):] == " ...": self.message = self.message[:-4]
			if self.message[-len("."):] == ".": self.message = self.message[:-1]
			if self.message[0].upper() != self.message[0]: self.message = self.message[1:]+self.message[0].upper()+self.message[1:]
			self.message = color.fill(self.message)
			self.index = index
			self.max = max
			self.progress = None
			self.last_message = None
			self.log_level = log_level
		def next(self, count=1, decimals=2):
			self.index += count
			p = round((self.index / self.max) * 100, decimals)
			if p != self.progress:
				self.progress = p
				self.last_message = f"{self.message} ... {self.progress}%"
				if self.log_level >= 0:
					print(self.last_message, end="\r")
		def stop(self, message=None, success=True, response=None):
			if self.log_level >= 0:
				if response == None:
					if message == None: message = self.message
				else:
					if response["error"] == None:
						message = response["message"]
					else:
						success = False
						message = "Error: "+response["error"]
				if self.last_message != None:
					print(self.__empty_message__(length=len(f"{self.last_message}")), end="\r")
				if success:
					print(f"{message} ... done")
				else:
					print(f"{message} ... {color.red}failed{color.end}")
		def __empty_message__(self, length=len("hello world")):
			s = ""
			for i in range(length): s += " "
			return s

	#

#


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# insert the package for universal imports.
def __get_source_path__(
	# the path (leave None to use self.path) (param #1).
	path=None,
	# the dirs back.
	back=1,
):
	base = path.replace('//','/')
	if base[len(base)-1] == '/': base = base[:-1]
	if len(base.split("/")) <= 1: raise ValueError("Path [{}] has no base.".format(base))
	startslash = True
	if base[0] != "/":
		startslash = False
	base = base.split("/")
	m, c, s = len(base), 0, ""
	for i in base:
		if c >= m-back: break
		if c == 0:
			s = f"/{i}/"
		else:
			s += f"{i}/"
		c += 1
	if startslash:
		return s
	else:
		return s[1:]
	###### OLD.
	base = path.replace('//','/')
	if len(base.split("/")) <= 1: raise ValueError("Path [{}] has no base.".format(base))
	if base[len(base)-1] == '/': base = base[:-1]
	for x in range(back, back+1):
		last = (base.split('/')[len(base.split('/'))-1]).replace('//','/')
		base = base[:-len("/"+last)]
	while True:
		if '//' in base: base = base.replace('//','/')
		else: break
	if base[len(base)-1] != "/": base += '/'
	return base
	"""splitted, result, count = path.split('/'), "", 0
	for i in splitted:
		if count < len(splitted) - 1 - back:
			result += '/' + i
		else: result += "/"
		count += 1
	"""
import sys ; sys.path.insert(1, __get_source_path__(__file__, back=2))

# imports.
from dev0s.classes.config import *
from dev0s import *

# the cli object class.
class CLI_(CLI.CLI):
	def __init__(self):
		
		# cli.
		CLI.CLI.__init__(self,
			modes={
				"Installation":"",
				"    --install":"Install the DevOS library.",
				"    --uninstall":"Uninstall the DevOS library.",
				"    --reinstall":"Reinstall the DevOS library.",
				"    --link":"Link (activate) the DevOS library.",
				"    --unlink":"Unlink (deactivate) the DevOS library.",
				"    --update":"Update the DevOS library.",
				"Defaults":"",
				"    --version":"Show the dev0s version.",
				"    -h / --help":"Show the documentation.",
			},
			options={
			},
			executable=__file__,
			alias=ALIAS,)
		s = self.documentation.split("\nAuthor:")
		before = s[1]
		self.documentation = s[0]
		self.documentation += "\nAdditional Libraries:"
		self.documentation += "\n    $ syst3m --help : Show the syst3m documentation."
		self.documentation += "\n    $ netw0rk --help : Show the netw0rk documentation."
		self.documentation += "\n    $ encrypti0n --help : Show the encrypti0n documentation."
		self.documentation += "\n    $ ssht00ls --help : Show the ssht00ls documentation."
		self.documentation += "\nAuthor:"+before

		#
	def start(self):
		
		# check arguments.
		self.arguments.check(exceptions=["--log-level", "--create-alias", "--version"], json=Defaults.options.json)

		#
		# BASICS
		#

		# help.
		if self.arguments.present(['-h', '--help']):
			self.docs(success=True, json=Defaults.options.json)

		# version.
		elif self.arguments.present(['--version']):
			print(f"{ALIAS} version:",Files.load(f"{SOURCE_PATH}/.version").replace("\n",""))

		# install.
		elif self.arguments.present('--install'):
			response = manager.installation.install()
			if response.message != None and "Successfully installed " not in response.message: Response.log(response=response)

		# uninstall.
		elif self.arguments.present('--uninstall'):
			response = manager.installation.uninstall()
			Response.log(response=response)

		# reinstall.
		elif self.arguments.present('--reinstall'):
			response = manager.installation.uninstall()
			Response.log(response=response)
			response = manager.installation.install()
			if response.message != None and "Successfully installed " not in response.message: Response.log(response=response)

		# link.
		elif self.arguments.present('--link'):
			response = manager.installation.link()
			Response.log(response=response)

		# unlink.
		elif self.arguments.present('--unlink'):
			response = manager.installation.unlink()
			Response.log(response=response)

		# invalid.
		else: self.invalid()

		#
	def invalid(self):
		print(self.documentation)
		print("Selected an invalid mode.")
		sys.exit(1)
	
# main.
if __name__ == "__main__":
	cli = CLI_()
	cli.start()
